/*
 * Copyright (c) 2021 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *                                           Anthony Mallet on Wed Sep  8 2021
 */
#ifndef H_XSENSMVN_CODELS
#define H_XSENSMVN_CODELS

#include <sys/socket.h>
#include <sys/time.h>

#include <stdint.h>
#include <string.h>

#include "xsensmvn_c_types.h"

struct xsensmvn_remote_s {
  int fd;
};

struct xsensmvn_stream_s {
  int fd;
};

/* return an absolute deadline for given ms timeout */
static inline struct timeval
mvn_deadline(int ms)
{
  struct timeval d;

  gettimeofday(&d, NULL);
  d.tv_usec += ms * 1000;
  if (d.tv_usec >= 1000000) {
    d.tv_usec -= 1000000;
    d.tv_sec++;
  }

  return d;
}

/* return a system error */
static inline genom_event
mvn_e_sys_error(int e, const char *s, genom_context self)
{
  xsensmvn_e_sys_detail d;
  size_t l = 0;

  d.code = e;
  if (s) {
    strncpy(d.what, s, sizeof(d.what) - 3);
    l = strlen(s);
    strcpy(d.what + l, ": ");
    l += 2;
  }
  if (strerror_r(d.code, d.what + l, sizeof(d.what) - l)) { /* ignore error*/; }
  return xsensmvn_e_sys(&d, self);
}

extern const char *mvnsegname[];
extern size_t const mvnsegnames;

int	mvn_remote_socket(const char *host, const char *port,
                struct timeval deadline, char *emsg, size_t emsgsize);
int	mvn_remote_localhost(int fd, char *host, size_t hostlen,
                char *emsg, size_t emsgsize);
int	mvn_remote_ping(int fd, struct timeval deadline);
int	mvn_remote_stream(int fd, bool add,
                const char *host, const char *port, const char *proto,
                struct timeval deadline);

int	mvn_stream_socket(const char *port, char *emsg, size_t emsgsize);
int	mvn_stream_gethost(char *host, size_t hostlen, char *port,
                size_t portlen, char *emsg, size_t emsgsize);
int	mvn_stream_decode(const void *data, size_t len,
                xsensmvn_body_s *bodies);

#endif /* H_XSENSMVN_CODELS */
